/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "NodeList.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/GdomHTMLCollection.h"
#include "gdom/GdomNode.h"
#include "gdom/GdomNodeList.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomHTMLCollection.h"
#include "gdom/PrivateGdomNode.h"
#include "gdom/PrivateGdomNodeList.h"
#include "gdom/PrivateIntGdomHTMLCollection.h"
#include "gdom/PrivateIntGdomNode.h"
#include "gdom/PrivateIntGdomNodeList.h"
#include "glib-object.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
} /* namespace WebCore */
extern "C" {

GDOMHTMLCollection::GDOMHTMLCollection(PassRefPtr<WebCore::HTMLCollection> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMHTMLCollection::~GDOMHTMLCollection()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}

GdomNode *
gdom_html_collection_item (GdomHTMLCollection *thiz, gulong index)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::HTMLCollection * item = (gdom_html_collection_get_private(thiz));
    g_return_val_if_fail (item, 0);
    PassRefPtr<WebCore::Node> g_res = WTF::getPtr(item->item(index));
    GdomNode * res = static_cast<GdomNode *>(WebCore::toGDOM(g_res.get()));
    return res;

}

GdomNode *
gdom_html_collection_named_item (GdomHTMLCollection *thiz, gchar * name)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::HTMLCollection * item = (gdom_html_collection_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (name, 0);
    WebCore::String _g_name = WebCore::String::fromUTF8(name);
    PassRefPtr<WebCore::Node> g_res = WTF::getPtr(item->namedItem(_g_name));
    GdomNode * res = static_cast<GdomNode *>(WebCore::toGDOM(g_res.get()));
    return res;

}

GdomNodeList *
gdom_html_collection_tags (GdomHTMLCollection *thiz, gchar * name)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::HTMLCollection * item = (gdom_html_collection_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (name, 0);
    WebCore::String _g_name = WebCore::String::fromUTF8(name);
    PassRefPtr<WebCore::NodeList> g_res = WTF::getPtr(item->tags(_g_name));
    GdomNodeList * res = static_cast<GdomNodeList *>(WebCore::toGDOM(g_res.get()));
    return res;

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "HTMLCollection.h"
#include <gdom/GdomHTMLCollection.h>
#include <gdom/PrivateGdomHTMLCollection.h>
#include <gdom/PrivateIntGdomHTMLCollection.h>
#include <gdom/GdomHTMLCollectionprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomHTMLCollection, gdom_html_collection, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomHTMLCollectionPrivate {
    class WebCore::HTMLCollection *item;
};

#define GDOM_HTML_COLLECTION_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_HTML_COLLECTION, GdomHTMLCollectionPrivate))
*/
G_END_DECLS
WebCore::HTMLCollection *
gdom_html_collection_get_private(GdomHTMLCollection *request)
{
    GDOMHTMLCollection *res;
    res = static_cast<GDOMHTMLCollection*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::HTMLCollection*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_LENGTH,
};


static void gdom_html_collection_finalize(GObject* object)
{
    /*GdomHTMLCollection* request = GDOM_HTML_COLLECTION(object);*/
    /*GdomHTMLCollectionPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_html_collection_parent_class)->finalize(object);
}

static void gdom_html_collection__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomHTMLCollection *self = GDOM_HTML_COLLECTION(object);
    WebCore::HTMLCollection * gself = (gdom_html_collection_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_html_collection__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomHTMLCollection *self = GDOM_HTML_COLLECTION(object);
    WebCore::HTMLCollection * gself = (gdom_html_collection_get_private(self));
    switch (prop_id) {
    case PROP_LENGTH:
    {
         g_value_set_ulong (value, gself->length());
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_html_collection_class_init(GdomHTMLCollectionClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_html_collection_finalize;
    g_klass->set_property = gdom_html_collection__set_property_fn;
    g_klass->get_property = gdom_html_collection__get_property_fn;

    g_object_class_install_property (g_klass, PROP_LENGTH,
            g_param_spec_ulong ("length", /* name */
                "html_collection_length", /* short description */
                "read-only  gulong HTMLCollection.length", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXULONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomHTMLCollectionPrivate));*/
}

static void gdom_html_collection_init(GdomHTMLCollection* request)
{
    /*GdomHTMLCollectionPrivate* priv = GDOM_HTML_COLLECTION_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

